@@ -12,6 +12,7 @@
 #include <limits.h>
 #include <ctype.h>
 #include <errno.h>
+#include <dirent.h>
 
 #include "pwquality.h"
 #include "pwqprivate.h"
@@ -32,6 +33,7 @@ pwquality_default_settings(void)
         pwq->up_credit = PWQ_DEFAULT_UP_CREDIT;
         pwq->low_credit = PWQ_DEFAULT_LOW_CREDIT;
         pwq->oth_credit = PWQ_DEFAULT_OTH_CREDIT;
+        pwq->dict_check = PWQ_DEFAULT_DICT_CHECK;
 
         return pwq;
 }
@@ -59,6 +61,7 @@ static const struct setting_mapping s_map[] = {
  { "maxclassrepeat", PWQ_SETTING_MAX_CLASS_REPEAT, PWQ_TYPE_INT},
  { "maxsequence", PWQ_SETTING_MAX_SEQUENCE, PWQ_TYPE_INT},
  { "gecoscheck", PWQ_SETTING_GECOS_CHECK, PWQ_TYPE_INT},
+ { "dictcheck", PWQ_SETTING_DICT_CHECK, PWQ_TYPE_INT},
  { "badwords", PWQ_SETTING_BAD_WORDS, PWQ_TYPE_STR},
  { "dictpath", PWQ_SETTING_DICT_PATH, PWQ_TYPE_STR}
 };
@@ -97,19 +100,14 @@ set_name_value(pwquality_settings_t *pwq, const char *name, const char *value)
 
 #define PWQSETTINGS_MAX_LINELEN 1023
 
-/* parse the configuration file (if NULL then the default one) */
+/* parse a single configuration file*/
 int
-pwquality_read_config(pwquality_settings_t *pwq, const char *cfgfile, void **auxerror)
+read_config_file(pwquality_settings_t *pwq, const char *cfgfile, void **auxerror)
 {
         FILE *f;
         char linebuf[PWQSETTINGS_MAX_LINELEN+1];
         int rv = 0;
 
-        if (auxerror)
-                *auxerror = NULL;
-        if (cfgfile == NULL)
-                cfgfile = PWQUALITY_DEFAULT_CFGFILE;
-
         f = fopen(cfgfile, "r");
         if (f == NULL) {
                 /* ignore non-existent default config file */
@@ -150,9 +148,11 @@ pwquality_read_config(pwquality_settings_t *pwq, const char *cfgfile, void **aux
                 if (*ptr == '\0')
                         continue;
 
+                eq = 0;
                 name = ptr;
                 while (*ptr != '\0') {
-                        if (isspace(*ptr)) {
+                        if (isspace(*ptr) || *ptr == '=') {
+                                eq = *ptr == '=';
                                 *ptr = '\0';
                                 ++ptr;
                                 break;
@@ -160,7 +160,6 @@ pwquality_read_config(pwquality_settings_t *pwq, const char *cfgfile, void **aux
                         ++ptr;
                 }
 
-                eq = 0;
                 while (*ptr != '\0') {
                         if (*ptr != '=' || eq) {
                                 if (!isspace(*ptr)) {
@@ -183,6 +182,86 @@ pwquality_read_config(pwquality_settings_t *pwq, const char *cfgfile, void **aux
         return rv;
 }
 
+static int
+filter_conf(const struct dirent *d)
+{
+        const char *p;
+
+        if ((p = strstr(d->d_name, ".conf")) == NULL)
+                return 0;
+
+        if (p[5] != '\0')
+                return 0;
+
+        return 1;
+}
+
+static int
+comp_func(const struct dirent **a, const struct dirent **b)
+{
+        return strcmp ((*a)->d_name, (*b)->d_name);
+}
+
+/* parse the configuration file (if NULL then the default one) */
+int
+pwquality_read_config(pwquality_settings_t *pwq, const char *cfgfile, void **auxerror)
+{
+        char *dirname;
+        struct dirent **namelist;
+        int n;
+        int i;
+        int rv = 0;
+
+        if (auxerror)
+                *auxerror = NULL;
+        if (cfgfile == NULL)
+                cfgfile = PWQUALITY_DEFAULT_CFGFILE;
+
+        /* read "*.conf" files from "<cfgfile>.d" directory first */
+
+        if (asprintf(&dirname, "%s.d", cfgfile) < 0)
+                return PWQ_ERROR_MEM_ALLOC;
+
+        /* we do not care about scandir races here so we use scandir */
+        n = scandir(dirname, &namelist, filter_conf, comp_func);
+
+        if (n < 0) {
+                namelist = NULL;
+
+                if (errno == ENOMEM) {
+                        free(dirname);
+                        return PWQ_ERROR_MEM_ALLOC;
+                } /* other errors are ignored */
+        }
+
+        for (i = 0; i < n; i++) {
+                char *subcfg;
+
+                if (rv) {
+                        free(namelist[i]);
+                        continue;
+                }
+
+                if (asprintf(&subcfg, "%s/%s", dirname, namelist[i]->d_name) < 0)
+                        rv = PWQ_ERROR_MEM_ALLOC;
+                else {
+                        rv = read_config_file(pwq, subcfg, auxerror);
+                        if (rv == PWQ_ERROR_CFGFILE_OPEN)
+                                rv = 0; /* ignore, this one does not modify auxerror */
+                        free(subcfg);
+                }
+
+                free(namelist[i]);
+        }
+        free(dirname);
+        free(namelist);
+
+        if (rv)
+                return rv;
+
+        return read_config_file(pwq, cfgfile, auxerror);
+}
+
 /* useful for setting the options as configured on a pam module
  * command line in form of <opt>=<val> */
 int
@@ -251,6 +330,9 @@ pwquality_set_int_value(pwquality_settings_t *pwq, int setting, int value)
         case PWQ_SETTING_GECOS_CHECK:
                 pwq->gecos_check = value;
                 break;
+        case PWQ_SETTING_DICT_CHECK:
+                pwq->dict_check = value;
+                break;
         default:
                 return PWQ_ERROR_NON_INT_SETTING;
         }
@@ -322,9 +404,15 @@ pwquality_get_int_value(pwquality_settings_t *pwq, int setting, int *value)
         case PWQ_SETTING_MAX_CLASS_REPEAT:
                 *value = pwq->max_class_repeat;
                 break;
+        case PWQ_SETTING_MAX_SEQUENCE:
+                *value = pwq->max_sequence;
+                break;
         case PWQ_SETTING_GECOS_CHECK:
                 *value = pwq->gecos_check;
                 break;
+        case PWQ_SETTING_DICT_CHECK:
+                *value = pwq->dict_check;
+                break;
         default:
                 return PWQ_ERROR_NON_INT_SETTING;
         }
@@ -349,8 +437,8 @@ pwquality_get_str_value(pwquality_settings_t *pwq, int setting, const char **val
 }
 
 /*
- * Copyright (c) Red Hat, Inc, 2011
- * Copyright (c) Tomas Mraz <tm@t8m.info>, 2011
+ * Copyright (c) Red Hat, Inc, 2011, 2015
+ * Copyright (c) Tomas Mraz <tm@t8m.info>, 2011, 2015
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
